package cn.iocoder.yudao.module.bpm.api.task;

import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.instance.BpmProcessInstanceCancelReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskRespVO;
import cn.iocoder.yudao.module.bpm.resp.TaskResp;
import cn.iocoder.yudao.module.bpm.service.task.BpmProcessInstanceService;
import cn.iocoder.yudao.module.bpm.service.task.BpmTaskService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

import static cn.iocoder.yudao.framework.web.core.util.WebFrameworkUtils.getLoginUserId;

/**
 * Flowable 流程实例 Api 实现类
 *
 * @author 捷道源码
 * @author jason
 */
@Service
@Validated
@Slf4j
public class BpmProcessInstanceApiImpl implements BpmProcessInstanceApi {

    @Resource
    private BpmProcessInstanceService processInstanceService;

    @Resource
    BpmTaskService bpmTaskService;

    @Override
    public String createProcessInstance(Long userId, @Valid BpmProcessInstanceCreateReqDTO reqDTO) {
        return processInstanceService.createProcessInstance(userId, reqDTO);
    }

    @Override
    public void cancelBmpById(Long userId, String bpmProcessInstanceId, String reason) {
        BpmProcessInstanceCancelReqVO cancelReqVO = new BpmProcessInstanceCancelReqVO();
        cancelReqVO.setId(bpmProcessInstanceId);
        cancelReqVO.setReason(reason);
        processInstanceService.cancelProcessInstance(userId, cancelReqVO);
    }

    @Override
    public TaskResp getLastTaskByProcessInstanceId(String processInstanceId) {
        List<BpmTaskRespVO> list = bpmTaskService.getTaskListByProcessInstanceId(processInstanceId);
        log.error("---+++++++++++++++---{}", JSONUtil.toJsonStr(list));
        if (null != list && list.size() > 0) {
            BpmTaskRespVO bpmTaskRespVO = list.get(0);
            TaskResp taskResp = new TaskResp();

            TaskResp.User user = new TaskResp.User();

            BpmTaskRespVO.User userResp = bpmTaskRespVO.getAssigneeUser();
            user.setId(userResp.getId());
            user.setDeptId(userResp.getDeptId());
            user.setNickname(userResp.getNickname());
            user.setDeptName(userResp.getDeptName());


            taskResp.setAssigneeUser(user);
            taskResp.setComment(bpmTaskRespVO.getComment());
            taskResp.setResult(bpmTaskRespVO.getResult());
            taskResp.setEndTime(bpmTaskRespVO.getEndTime());
            taskResp.setDurationInMillis(bpmTaskRespVO.getDurationInMillis());

            return taskResp;
        }
        return null;
    }


    @Override
    public Long getTodoTaskCount(Long userId) {
        return bpmTaskService.getTodoTaskCount(getLoginUserId());
    }

    @Override
    @Async
    public void approveTask(String id, String comment) {
        BpmTaskApproveReqVO bpmTaskApproveReqVO = new BpmTaskApproveReqVO();
        bpmTaskApproveReqVO.setId(id);
        bpmTaskApproveReqVO.setComment(comment);
        bpmTaskService.approveTask(1L, bpmTaskApproveReqVO);
    }
}
